// Random access example
// This code is experimental. It reads ProMC and tries to write HEPMC file.
// Author: Sergei Chekanov (ANL) 

#include <cstdlib>
#include <string>
#include <cassert>
#include <iostream>
#include "ProMC.pb.h"
#include "ProMCBook.h"
#include "ProMCHeader.pb.h"

using namespace std;



// main program to read ProMC and convert to HEPMC
int main(int argc, char** argv) {

  // Look for a help argument
  for (int i = 1; i < argc; ++i) {
    const string arg = argv[i];
    if (arg == "--help" || arg == "-h") {
      cout << argv[0] << ": Random access example" << endl;
      cout << "Usage: " << argv[0] << "[<input ProMC file>]" << endl;
      exit(0);
    }
  }

  // Choose input and output files from the command line
  string infile("-"); 
  if (argc == 2) {
    infile = argv[1];
  } else if (argc != 2) {
    cerr << "Usage: " << argv[0] << "[<PROMC files input>]" << endl;
    exit(1);
  }

 // -------------- ProMC file ----------------------
  ProMCBook *epbook = new ProMCBook( infile.c_str(),"r");
  // get the version number
  int  h=epbook->getVersion();
  // cout << "ProMC version = " << h << endl;
  // get the description of this file
  //string d=epbook->getDescription();
  //cout << "Description = " << d << endl;
  unsigned int  nev=epbook->getEvents();
  cout << "Events = " << nev  << endl;

  // get the header file with units, cross sections etc.
  ProMCHeader header = epbook->getHeader();
  //cout << "Cross section=" << header.cross_section() << endl;
  // here is the units for energy
  float kEV=(float)(header.momentumunit());
  float kLe=(float)(header.lengthunit()); 
  cout << "Energy unit=" << kEV << endl;
  cout << "Length unit=" << kLe << endl;
 

 
  // random access 
  ProMCEvent eve1 = epbook->event(0);
  ProMCEvent_Event  *e1=eve1.mutable_event();
  int event_number=e1->number();
  ProMCEvent_Particles  *pa1=eve1.mutable_particles();
  cout << "Read event=" << event_number << " with Nr particles=" <<  pa1->pdg_id_size() << endl;


  ProMCEvent eve2 = epbook->event(nev-1);
  ProMCEvent_Event  *e2=eve2.mutable_event();
  int event_number2=e2->number();
  ProMCEvent_Particles  *pa2=eve2.mutable_particles();
  cout << "Read event=" << event_number2 << " with Nr particles=" <<  pa2->pdg_id_size() << endl;



 
  epbook->close(); // close

  return EXIT_SUCCESS;
}
